iSpring = {};
iSpring.Logger = {};
iSpring.LMSApiWrapper = {};
iSpring.FinalizeQuizHandlers = {};
iSpring.SetQuizScoreHandlers = {};
iSpring.LMS = {};


// Utils
CUtils = function() { }
CUtils.__name__ = "CUtils";
CUtils.convertToInt = function(value)
{
	if (value < 0)
	{
		return Math.ceil(value);
	}
	return Math.floor(value);
}
CUtils.prototype.__class__ = CUtils;


// Time formatting
CTimeFormat = function() { }
CTimeFormat.__name__ = "CTimeFormat";
CTimeFormat.formatDuration = function(duration)
{
	var s = duration;
	var h = CUtils.convertToInt(s / 3600);
	s -= h * 3600;
	var m = CUtils.convertToInt(s / 60);
	s -= m * 60;
	return CTimeFormat.intToStr(h) + ":" + CTimeFormat.intToStr(m) + ":" + CTimeFormat.intToStr(s);
}
CTimeFormat.intToStr = function(value)
{
	var data = "";
	if (value < 10)
	{
		data += "0";
	}
	data += value.toString();
	return data;
}
CTimeFormat.prototype.__class__ = CTimeFormat;


// Time controller
iSpring.TimeController = function(duration, interval, listener)
{
	this.m_duration = duration;
	this.m_listener = listener;
	this.m_window = window;
	this.m_interval = interval;
	this.m_time = new Date().getTime();

	var thisPtr = this;
	function onTimeout()
	{
		thisPtr.onEndTime();
	}
	function onTimer()
	{
		thisPtr.onTime();
	}

	this.m_timeoutId = this.m_window.setTimeout(onTimeout, this.m_duration * 1000);
	this.m_intervalId = this.m_window.setInterval(onTimer, this.m_interval * 1000);
	this.m_work = true;
}
iSpring.TimeController.prototype.m_duration = null;
iSpring.TimeController.prototype.m_interval = null;
iSpring.TimeController.prototype.m_intervalId = null;
iSpring.TimeController.prototype.m_listener = null;
iSpring.TimeController.prototype.m_time = null;
iSpring.TimeController.prototype.m_timeoutId = null;
iSpring.TimeController.prototype.m_window = null;
iSpring.TimeController.prototype.m_work = null;
iSpring.TimeController.prototype.__name__ = "iSpring.TimeController";
iSpring.TimeController.prototype = iSpring.TimeController;
iSpring.TimeController.prototype.onEndTime = function()
{
	if (this.m_work)
	{
		this.m_listener.onEndTime();
	}
}
iSpring.TimeController.prototype.onTime = function()
{
	if (this.m_work)
	{
		var duration = new Date().getTime() - this.m_time;
		this.m_listener.onTime(CUtils.convertToInt(duration / 1000));
	}
}
iSpring.TimeController.prototype.stopTimers = function()
{
	if (this.m_work)
	{
		this.m_window.clearInterval(this.m_intervalId);
		this.m_window.clearTimeout(this.m_timeoutId);
		this.m_work = false;
	}
}


// Logger
iSpring.Logger = function() { }
iSpring.Logger.prototype.__name__ = "iSpring.Logger";
iSpring.Logger.prototype = iSpring.Logger;
iSpring.Logger.prototype.log = function(str)
{
	var textArea = document.getElementById("iSpring_log");
	if (textArea)
	{
		textArea.value = textArea.value + str + "\n";
	}
}
iSpring.Logger.prototype.logError = function(str)
{
	this.log("ERROR: " + str);
}

// LMS API Wrapper
iSpring.LMSApiWrapper = function(version)
{
	this.m_version = version;
	this.m_apiHandle = null;
	this.m_findAPITries = 255;
	this.m_noAPIFound = false;
	this.m_terminated = true;
	this.m_logger = new iSpring.Logger();
}

iSpring.LMSApiWrapper.SCORM_VERSION_12 = "1.2";
iSpring.LMSApiWrapper.SCORM_VERSION_2004 = "2004";
iSpring.LMSApiWrapper.AICC_VERSION = "aicc";

iSpring.LMSApiWrapper.prototype.__name__ = "iSpring.LMSApiWrapper";
iSpring.LMSApiWrapper.prototype.m_version = null;
iSpring.LMSApiWrapper.prototype.m_apiHandle = null;
iSpring.LMSApiWrapper.prototype.m_findAPITries = null;
iSpring.LMSApiWrapper.prototype.m_noAPIFound = null;
iSpring.LMSApiWrapper.prototype.m_terminated = null;
iSpring.LMSApiWrapper.prototype.m_logger = null;
iSpring.LMSApiWrapper.prototype = iSpring.LMSApiWrapper;
iSpring.LMSApiWrapper.prototype.getWindowApi = function(win)
{
	switch (this.m_version)
	{
		case this.SCORM_VERSION_12:
		case this.AICC_VERSION:
			return win.API;
		case this.SCORM_VERSION_2004:
			return win.API_1484_11;
	}
}

iSpring.LMSApiWrapper.prototype.findAPI = function(win)
{
	while ((this.getWindowApi(win) == null) && (win.parent != null) && (win.parent != win))
	{
		this.m_findAPITries--;
		if (this.m_findAPITries < 0)
		{
			return null;
		}
		win = win.parent;
	}
	return this.getWindowApi(win);
}

iSpring.LMSApiWrapper.prototype.getAPI = function()
{
	var win = window;
	var theAPI = this.findAPI(win);
	if ((theAPI == null) && (win.opener != null))
	{
		theAPI = this.findAPI(win.opener);
	}
	if (theAPI == null)
	{
		this.m_noAPIFound = true;
	}
	return theAPI;
}

iSpring.LMSApiWrapper.prototype.getApiHandle = function()
{
	if (this.m_apiHandle == null)
	{
		if (this.m_noAPIFound == false)
		{
			this.m_apiHandle = this.getAPI();
		}
	}
	if (!this.m_apiHandle)
	{
		this.m_logger.logError("lms api not found");
	}
	return this.m_apiHandle;
}

iSpring.LMSApiWrapper.prototype.commit = function()
{
	if (this.m_terminated == false)
	{
		var api = this.getApiHandle();
		if (api != null)
		{
			var result = false;
			switch (this.m_version)
			{
				case this.SCORM_VERSION_12:
				case this.AICC_VERSION:
					result = this.strToBool(api.LMSCommit(""));
					break;
				case this.SCORM_VERSION_2004:
					result = this.strToBool(api.Commit(""));
					break;
			}
			if (!result)
			{
				this.m_logger.logError("commit failed");
			}
		}
	}
}

iSpring.LMSApiWrapper.prototype.getValue = function(paramName)
{
	if (this.m_terminated == false)
	{
		var api = this.getApiHandle();

		if (api != null)
		{
			switch (this.m_version)
			{
				case this.SCORM_VERSION_12:
				case this.AICC_VERSION:
					return api.LMSGetValue(paramName);
				case this.SCORM_VERSION_2004:
					return api.GetValue(paramName);
			}
		}
	}
	return "";
}

iSpring.LMSApiWrapper.prototype.initialize = function()
{
	if (this.m_terminated == true)
	{
		var api = this.getApiHandle();
		if (api != null)
		{
			var result = false;
			switch (this.m_version)
			{
				case this.SCORM_VERSION_12:
				case this.AICC_VERSION:
					result = this.strToBool(api.LMSInitialize(""));
					break;
				case this.SCORM_VERSION_2004:
					result = this.strToBool(api.Initialize(""));
					break;
			}

			if (!result)
			{
				this.m_logger.logError("initialization failed");
			}
			this.m_terminated = false;
		}
	}
}

iSpring.LMSApiWrapper.prototype.setValue = function(paramName, value)
{
	if (this.m_terminated == false)
	{
		var api = this.getApiHandle();
		if (api != null)
		{
			if (typeof(value) == "number" && this.m_version != this.SCORM_VERSION_2004) 
			{
				//AICC and SCORM12 requires only 2 digits after dot
				value = Math.round(value * 100) / 100;
			}
			var result = false;
			paramName = paramName + "";
			value = value + "";

			switch (this.m_version)
			{
				case this.SCORM_VERSION_12:
				case this.AICC_VERSION:
					result = this.strToBool(api.LMSSetValue(paramName, value));
					break;
				case this.SCORM_VERSION_2004:
					result = this.strToBool(api.SetValue(paramName, value));
					break;
			}
			if (!result)
			{
				this.m_logger.logError("setValue failed - name = " + paramName + ", value = " + value);
			}
		}
	}
}

iSpring.LMSApiWrapper.prototype.terminate = function()
{
	if (this.m_terminated == false)
	{
		var api = this.getApiHandle();
		if (api != null)
		{
			var result = false;
			switch (this.m_version)
			{
				case this.SCORM_VERSION_12:
				case this.AICC_VERSION:
					result = this.strToBool(api.LMSFinish(""));
					break;
				case this.SCORM_VERSION_2004:
					result = this.strToBool(api.Terminate(""));
					break;
			}
			if (!result)
			{
				this.m_logger.logError("terminate failed");
			}
			this.m_terminated = true;
		}
	}
}

iSpring.LMSApiWrapper.strToBool = function(str)
{
	switch (str)
	{
		case "true":
		case "1":
		case "yes":
			return true;
		case "false":
		case "0":
		case "no":
		case null:
			return false;
		default:
			return Boolean(str);
	}
}


// iSpring LMS
iSpring.LMS = function(id, version, movie, maxTime,
		exitOnTimeout, showMessageOnTimeout, messageOnTimeout, 
		quizzes, rateSlidesWeight, rateSlidesCount,
		totalGrade, passingScore)
{
	this.m_id = id;
	this.m_version = version;
	this.m_movie = movie;
	
	this.m_maxTime = maxTime;
	this.m_exitOnTimeout = exitOnTimeout;
	this.m_showMessageOnTimeout = showMessageOnTimeout;
	this.m_messageOnTimeout = messageOnTimeout;
	
	this.m_ratedQuizzes  = quizzes;
	this.m_quizScores = [];
	
	this.m_rateSlidesWeight = rateSlidesWeight;
	this.m_rateSlidesCount  = rateSlidesCount;
	
	this.m_totalScore   = totalGrade;
	this.m_passingScore = passingScore;
	
	this.m_slidesNum = this.m_movie.presentation_getSlidesCount();
	
	this.m_wrapper = new iSpring.LMSApiWrapper(this.m_version);
	this.setQuizHandlers();

	this.m_suspendData = new Array;
	this.m_suspendData.push(true);
	if (this.m_slidesNum > 1)
	{
		for (i = 1; i < this.m_slidesNum; i++)
		{
			this.m_suspendData.push(false);
		}
	}
	this.m_lmsOpened = false;
	this.m_currentSlideIndex = 0;

	this.openLms();
}

iSpring.LMS.SCORM_VERSION_12 = "1.2";
iSpring.LMS.SCORM_VERSION_2004 = "2004";
iSpring.LMS.AICC_VERSION = "aicc";

iSpring.LMS.prototype.__name__ = "iSpring.LMS";
iSpring.LMS.prototype.m_id = null;
iSpring.LMS.prototype.m_movie = null;

iSpring.LMS.prototype.m_maxTime = null;
iSpring.LMS.prototype.m_exitOnTimeout = null;
iSpring.LMS.prototype.m_showMessageOnTimeout = null;
iSpring.LMS.prototype.m_messageOnTimeout = null;

iSpring.LMS.prototype.m_ratedQuizzes  = null;
iSpring.LMS.prototype.m_quizScores = null;
iSpring.LMS.prototype.m_rateSlidesWeight = null;
iSpring.LMS.prototype.m_rateSlidesCount  = null;

iSpring.LMS.prototype.m_totalScore   = null;
iSpring.LMS.prototype.m_passingScore = null;

iSpring.LMS.prototype.m_slidesNum = null;

iSpring.LMS.prototype.m_wrapper = null;
iSpring.LMS.prototype.m_lmsOpened = null;
iSpring.LMS.prototype.m_currentSlideIndex = null;

iSpring.LMS.prototype.m_suspendData = null;
iSpring.LMS.prototype.m_timeController = null;
iSpring.LMS.prototype.m_version = null;

iSpring.LMS.prototype = iSpring.LMS;

iSpring.LMS.prototype.getMovie = function()
{
	return this.m_movie;
}

iSpring.LMS.prototype.openLms = function()
{
	if (!this.m_lmsOpened)
	{
		this.m_wrapper.initialize();
		this.m_lmsOpened = true;
		
		if (this.m_ratedQuizzes.length > 0 || this.m_rateSlidesCount > 0)
		{
			this.setMinMaxScore(0, this.m_totalScore);
		}

		this.m_timeController = new iSpring.TimeController(this.m_maxTime, 15, this);
		this.restoreSuspendedData();
		
		this.updateLearnerProgress();
	}
}

iSpring.LMS.prototype.getId = function()
{
	return this.m_id;
}

iSpring.LMS.prototype.closeLms = function()
{
	if (this.m_lmsOpened)
	{
		this.m_timeController.stopTimers();
		if ( this.getCompletionStatus() )
		{
			this.setExitNormal();
		}
		else
		{
			this.setExitSuspend();
		}
		this.saveData();
		this.m_wrapper.terminate();
		this.m_lmsOpened = false;
	}
}

iSpring.LMS.prototype.restoreSuspendedData = function()
{
	var sData = this.getSuspendData().split(",");

	if (sData.length == this.m_suspendData.length)
	{
		for (var i = 0; i < sData.length; ++i)
		{
			this.m_suspendData[i] = (sData[i] == 'true');
		}
	}
}

iSpring.LMS.prototype.getMaxWeightScore = function()
{
	var maxScore = 0;
	
	var countQuizzes = this.m_ratedQuizzes.length;
	for (var i = 0; i < countQuizzes; ++i)
	{
		maxScore += this.m_ratedQuizzes[i].weight;
	}
	
	if (this.m_rateSlidesCount > 0)
	{
		maxScore += this.m_rateSlidesWeight;
	}
	
	return maxScore;
}

iSpring.LMS.prototype.getQuizzesWeightScore = function()
{
	var weightScore = 0;
	
	var countRatedQuizzes = this.m_ratedQuizzes.length;
	for (var i = 0; i < countRatedQuizzes; ++i)
	{
		var quizInfo = this.m_ratedQuizzes[i];
		
		var quizIndex = quizInfo.index;
		var weight    = quizInfo.weight;
		
		if (this.m_quizScores[quizIndex])
		{
			weightScore += this.m_quizScores[quizIndex] * weight;			
		}
	}
	
	return weightScore;
}

iSpring.LMS.prototype.getSlidesWeightScore = function()
{
	if (this.m_rateSlidesCount > 0)
	{
		return this.getViewedSlideCount() / this.m_slidesNum * this.m_rateSlidesWeight;
	}
	return 0;
}

iSpring.LMS.prototype.updateLearnerProgress = function()
{
	this.updateCompletionStatus();	
	this.updatePassedStatus();
}

iSpring.LMS.prototype.updateCompletionStatus = function()
{
	var countViewedSlidesForCompletion = (this.m_rateSlidesCount == 0 ? 
		this.m_slidesNum : this.m_rateSlidesCount);
    var slidesViewed = (countViewedSlidesForCompletion <= this.getViewedSlideCount());
	
	var quizzesViewed = true;
	for (var i = 0; i < this.m_ratedQuizzes.length; ++i)
	{
		var quizIndex = this.m_ratedQuizzes[i].index;
		if ( this.m_quizScores[quizIndex] == undefined )
		{
			quizzesViewed = false;
			break;
		}
	}
	
	this.setCompletionStatus(slidesViewed && quizzesViewed);
}

iSpring.LMS.prototype.updatePassedStatus = function()
{
	if (this.m_rateSlidesCount > 0 || this.m_ratedQuizzes.length > 0)
	{
		var quizzesScore = this.getQuizzesWeightScore();
		var slidesScore  = this.getSlidesWeightScore();
		
		var gainedWeightScore = quizzesScore + slidesScore;
		var maxWeightScore = this.getMaxWeightScore();
		
		var percentScore = gainedWeightScore / maxWeightScore;
		var gainedScore  = percentScore * this.m_totalScore;
		
		this.setScaledScore(percentScore);
		this.setRawScore(gainedScore);
		
		var delta = this.m_totalScore * 0.000001;
		this.setPassedStatus(gainedScore + delta >= this.m_passingScore);
	}	
	else if (this.m_version == this.SCORM_VERSION_2004)
	{
		this.setPassedStatus(undefined);
	}
}

iSpring.LMS.prototype.saveData = function()
{
	this.setSuspendData(this.m_suspendData.join(","));
	this.m_wrapper.commit();
}

iSpring.LMS.prototype.onCurrentSlideIndexChanged = function(slideIndex)
{
	if (this.m_lmsOpened)
	{
		this.m_currentSlideIndex = slideIndex;
		this.m_suspendData[slideIndex] = true;
		
		this.updateLearnerProgress();
	}
}

iSpring.LMS.prototype.convertToInt = function(value)
{
	if (value < 0)
	{
		return Math.ceil(value);
	}
	return Math.floor(value);
}

iSpring.LMS.prototype.formatTime = function(value)
{
	var h;
	var m;
	var s = value;
	h = this.convertToInt(s / 3600);
	s -= h * 3600;
	m = this.convertToInt(s / 60);
	s -= m * 60;

	switch (this.m_version)
	{
		case this.SCORM_VERSION_12:
		case this.AICC_VERSION:
			return this.format2(h) + ":" + this.format2(m) + ":" + this.format2(s);
		case this.SCORM_VERSION_2004:
			return "PT" + h + "H" + m + "M" + s + "S";
	}
}

iSpring.LMS.prototype.format2 = function(value)
{
	var str = "";
	if (value < 10)
	{
		str += "0";
	}
	return str + value.toString();
}

iSpring.LMS.prototype.getCompletionStatus = function()
{
	var p = this.selectParamVersion("cmi.core.lesson_status", "cmi.completion_status");
	var completionStatus = this.m_wrapper.getValue(p);
	return (completionStatus == "completed" || completionStatus == "passed");
}

iSpring.LMS.prototype.setExitNormal = function()
{
	switch (this.m_version)
	{
		case this.SCORM_VERSION_12:
		case this.AICC_VERSION:
			this.m_wrapper.setValue("cmi.core.exit", "");
			break;
		case this.SCORM_VERSION_2004:
			this.m_wrapper.setValue("cmi.exit", "normal");
			this.m_wrapper.setValue("adl.nav.request", "exitAll");
			break;
	}
}

iSpring.LMS.prototype.setExitSuspend = function()
{
	switch (this.m_version)
	{
		case this.SCORM_VERSION_12:
		case this.AICC_VERSION:
			this.m_wrapper.setValue("cmi.core.exit", "suspend");
			break;
		case this.SCORM_VERSION_2004:
			this.m_wrapper.setValue("cmi.exit", "suspend");
			this.m_wrapper.setValue("adl.nav.request", "exitAll");
			break;
	}
}

iSpring.LMS.prototype.setCompletionStatus = function(completed)
{
	var paramName  = this.selectParamVersion("cmi.core.lesson_status", "cmi.completion_status");
	var paramValue = (completed ? "completed" : "incomplete");

	this.m_wrapper.setValue(paramName, paramValue);
}

iSpring.LMS.prototype.setPassedStatus = function(passed)
{
	var paramName  = this.selectParamVersion("cmi.core.lesson_status", "cmi.success_status");
    var paramValue = "unknown";
    if (passed !== undefined)
    {
    	var paramValue = (passed ? "passed" : "failed");
    }
	
	this.m_wrapper.setValue(paramName, paramValue);
}

iSpring.LMS.prototype.setTime = function(value)
{
	var p = this.selectParamVersion("cmi.core.session_time", "cmi.session_time");
	this.m_wrapper.setValue(p, this.formatTime(value));
}

iSpring.LMS.prototype.getSuspendData = function()
{
	return this.m_wrapper.getValue("cmi.suspend_data");
}

iSpring.LMS.prototype.setSuspendData = function(data)
{
	this.m_wrapper.setValue("cmi.suspend_data", data);
}

iSpring.LMS.prototype.getTimeLimitActions = function()
{
	return "";
}

iSpring.LMS.prototype.setQuizFinalizeHandler = function()
{
	var thisPtr = this;
	this.m_movie.setListener("scorm_onQuizFinished", "iSpring.FinalizeQuizHandlers." + this.m_id);
	iSpring.FinalizeQuizHandlers[this.m_id] = function()
	{
		thisPtr.onFinalizeQuiz();
	}
}

iSpring.LMS.prototype.onFinalizeQuiz = function()
{
	// override
}

iSpring.LMS.prototype.setQuizScoreHandler = function()
{
	var thisPtr = this;
	this.m_movie.setListener("scorm_setQuizScore", "iSpring.SetQuizScoreHandlers." + this.m_id);
	iSpring.SetQuizScoreHandlers[this.m_id] = function(pointScore, percentScore, totalScore, totalScoreNormalized, passed)
	{
		thisPtr.setQuizScore(pointScore, percentScore, totalScore, totalScoreNormalized, passed);
	}
}

iSpring.LMS.prototype.setQuizScore = function(pointScore, percentScore, totalScore, totalScoreNormalized, passed)
{
	this.m_quizScores[this.m_currentSlideIndex] = percentScore;

	this.updateLearnerProgress();
}

iSpring.LMS.prototype.setQuizHandlers = function()
{
	this.setQuizFinalizeHandler();
	this.setQuizScoreHandler();
}

iSpring.LMS.prototype.setRawScore = function(score)
{
	var p = this.selectParamVersion("cmi.core.score.raw", "cmi.score.raw");
	this.m_wrapper.setValue(p, score);
}

iSpring.LMS.prototype.setMinMaxScore = function(min, max)
{
	var p1 = this.selectParamVersion("cmi.core.score.min", "cmi.score.min");
	var p2 = this.selectParamVersion("cmi.core.score.max", "cmi.score.max");
	this.m_wrapper.setValue(p1, min);
	this.m_wrapper.setValue(p2, max);
}

iSpring.LMS.prototype.setScaledScore = function(score)
{
	switch (this.m_version)
	{
		case this.SCORM_VERSION_12:
		case this.AICC_VERSION:
			break;
		case this.SCORM_VERSION_2004:
			this.m_wrapper.setValue("cmi.score.scaled", score);
			break;
	}
}

iSpring.LMS.prototype.selectParamVersion = function(scorm12Param, scorm2004Param)
{
	switch (this.m_version)
	{
		case this.SCORM_VERSION_12:
		case this.AICC_VERSION:
			return scorm12Param;
		case this.SCORM_VERSION_2004:
			return scorm2004Param;
	}
}

iSpring.LMS.prototype.getViewedSlideCount = function()
{
	var count = 0;
	for (i = 0; i < this.m_slidesNum; i++)
	{
		if (this.m_suspendData[i])
		{
			count++;
		}		
	}
	return count;
}

iSpring.LMS.prototype.onEndTime = function()
{
	if (this.m_showMessageOnTimeout)
	{
		alert(this.m_messageOnTimeout);
	}
	if (this.m_exitOnTimeout && this.m_lmsOpened)
	{
		this.setExitNormal();
		this.m_wrapper.terminate();
		this.m_lmsOpened = false;
	}
}

iSpring.LMS.prototype.onTime = function(time)
{
	this.setTime(time);
}
